-- Copyright  2021 Anton Zhuchkov

local pathcache
local win = ide.osname == "Windows"
local mac = ide.osname == "Macintosh"

return {
  name = "RE:INSTEAD",
  description = "RE:INSTEAD Metaparser Framework",
  api = {"baselib", "instead", "metaparser"},
  frun = function(self,wfilename,rundebug)
    local instead = ide.config.path.instead or pathcache -- check if the path is configured
    local projdir = self:fworkdir(wfilename)
    if not instead then
      local sep = win and ';' or ':'
      local default =
           win and (GenerateProgramFilesPath('Games/INSTEAD', sep)..sep)
        or mac and ('/Applications/love.app/Contents/MacOS'..sep)
        or ''
      local path = default
                 ..(os.getenv('PATH') or '')..sep
                 ..(GetPathWithSep(projdir))..sep
                 ..(os.getenv('HOME') and GetPathWithSep(os.getenv('HOME'))..'bin' or '')
      local paths = {}
      for p in path:gmatch("[^"..sep.."]+") do
        instead = instead or GetFullPathIfExists(p, win and 'reinstead.exe' or 'reinstead')
        table.insert(paths, p)
      end
      if not instead then
        ide:Print("Can't find reinstead executable in any of the following folders: "
          ..table.concat(paths, ", "))
        return
      end
      pathcache = instead
    end

    if not GetFullPathIfExists(projdir, 'main3.lua') then
      local altpath = wfilename:GetPath(wx.wxPATH_GET_VOLUME)
      local altname = GetFullPathIfExists(altpath, 'main.lua')
      if altname and wx.wxMessageBox(
          ("Can't find 'main3.lua' file in the current project folder.\n"
           .."Would you like to switch the project directory to '%s'?"):format(altpath),
          "INSTEAD Interpreter",
          wx.wxYES_NO + wx.wxCENTRE, ide:GetMainFrame()) == wx.wxYES then
        ide:SetProject(altpath)
        projdir = altpath
      else
        ide:Print(("Can't find 'main3.lua' file in the current project folder: '%s'.")
          :format(projdir))
        return
      end
    end
    
    local autoscript = GetFullPathIfExists(projdir, 'autoscript.txt');
     

    if rundebug then
      ide:GetDebugger():SetOptions({runstart = ide.config.debugger.runonstart ~= false})
    end

    -- suppress hiding ConsoleWindowClass as this is used by Love console
    local uhw = ide.config.unhidewindow
    local cwc = uhw and uhw.ConsoleWindowClass
    if uhw then uhw.ConsoleWindowClass = 0 end

    local params = self:GetCommandLineArg()
    local cmd = ('"%s" "%s"%s%s%s'):format(instead, projdir,
      params and " "..params or "", rundebug and ' -debug' or '', 
      autoscript and ' -i autoscript.txt' or '')
    -- CommandLineRun(cmd,wdir,tooutput,nohide,stringcallback,uid,endcallback)
    return CommandLineRun(cmd,projdir,true,true,nil,nil,
      function() if uhw then uhw.ConsoleWindowClass = cwc end end)
  end,
  hasdebugger = true,
  scratchextloop = true,
  takeparameters = true,
}
